import React, {FC, useEffect, useRef, useState, FormEvent} from 'react'
import {Step1} from './steps/Step1'
import {Step2} from './steps/Step2'
import {Step3} from './steps/Step3'
import {Step4} from './steps/Step4'
import {Step5} from './steps/Step5'
import {KTSVG} from '../../../../_metronic/helpers'
import {StepperComponent} from '../../../../_metronic/assets/ts/components'
import {Formik, Form, FormikValues} from 'formik'
import {createAccountSchemas, ICreateAccount, inits} from './CreateAccountWizardHelper'
import MarkdownRenderer from './MarkdownRenderer';
import io from 'socket.io-client';

const socket = io('http://localhost:8082'); // Replace with your server URL

const GptNewProject: FC = () => {

  const [selectedFile, setSelectedFile] = useState<File | null>(null);
  const [aiResponse, setAiResponse] = useState<string>('');
  const [aiLoadingText, setAiLoadingText] = useState<string>('');
  const [initGenerationResponse, setInitGenerationResponse] = useState<string>('');

  const handleFileSelect = (event: React.ChangeEvent<HTMLInputElement>) => {
    const file = event.target.files ? event.target.files[0] : null;
    setSelectedFile(file);
    setAiResponse('');
    setAiLoadingText('');
    setInitGenerationResponse('New file selected. Ready to submit.');
  };

  const handleSubmit = () => {
    if (selectedFile) {
      const reader = new FileReader();

      reader.onload = function(event) {
        const fileContent = event.target?.result as string;

        // Construct the JSON structure
        const klokGptGenerationRequest = {
          prompt: fileContent,
          typeOfGeneration: "new_offramp",
        };

        console.log("Data to send:", klokGptGenerationRequest);

        // Sending a POST request to the server
        fetch('http://localhost:8080/api/klokgpt/generate/v1', {
          method: 'POST',
          headers: {
            'Content-Type': 'application/json',
          },
          body: JSON.stringify(klokGptGenerationRequest),
        })
          .then(response => response.json())
          .then(data => {
            console.log('Response from server:', data);

            // Update the responses in the UI
            setInitGenerationResponse(new Date() + "\n" + JSON.stringify(data, null, 2));
            setAiResponse(''); // Clear the previous response (if any)
            setAiLoadingText('Generation process starting...');

            socket.emit('leave_all_rooms');
            socket.emit('join_room', data.id);
          })
          .catch(error => {
            console.error('Error:', error);
            setAiResponse('An error occurred during the generation process.');
          });
      };

      reader.readAsText(selectedFile); // Read the file content as text

      // Update UI to indicate that the process is starting
      setAiResponse('');
      setInitGenerationResponse('Calling the server... Initiating the generation process...');
    } else {
      alert('Please select a file first.');
    }
  };

  // Use useEffect to listen to socket messages
  useEffect(() => {
    socket.on('AIResponseToken', (message: string) => {
      console.log('Received message:', message);
      setAiResponse((prevResponse) => prevResponse + message); // Update aiResponse with new token
      setAiLoadingText('');
    });

    return () => {
      socket.off('AIResponseToken'); // Cleanup to avoid memory leaks
    };
  }, []);








  const stepperRef = useRef<HTMLDivElement | null>(null)
  const stepper = useRef<StepperComponent | null>(null)
  const [currentSchema, setCurrentSchema] = useState(createAccountSchemas[0])
  const [initValues] = useState<ICreateAccount>(inits)
  const [isSubmitButton, setSubmitButton] = useState(false)

  const loadStepper = () => {
    stepper.current = StepperComponent.createInsance(stepperRef.current as HTMLDivElement)
  }

  const prevStep = () => {
    if (!stepper.current) {
      return
    }

    setSubmitButton(stepper.current.currentStepIndex === stepper.current.totatStepsNumber! - 1)

    stepper.current.goPrev()

    setCurrentSchema(createAccountSchemas[stepper.current.currentStepIndex - 1])
  }

  const submitStep = (values: ICreateAccount, actions: FormikValues) => {
    if (!stepper.current) {
      return
    }

    setSubmitButton(stepper.current.currentStepIndex === stepper.current.totatStepsNumber! - 1)

    setCurrentSchema(createAccountSchemas[stepper.current.currentStepIndex])

    if (stepper.current.currentStepIndex !== stepper.current.totatStepsNumber) {
      stepper.current.goNext()
    } else {
      stepper.current.goto(1)
      actions.resetForm()
    }
  }

  useEffect(() => {
    if (!stepperRef.current) {
      return
    }

    loadStepper()
  }, [stepperRef])


  return (
    <div className='card'>
      <div className='card-body'>
        <div>
          <div>
            <pre id="initGenerationResponse" className="mt-3">
              {initGenerationResponse}
            </pre>
            <div className="form-group">
              <label htmlFor="fileInput" className="form-label">Upload your file</label>
              <input
                type="file"
                id="fileInput"
                className="form-control"
                onChange={handleFileSelect}
              />
            </div>
            <button
              id="sendRequest"
              className="btn btn-primary mt-3"
              onClick={handleSubmit}
              disabled={!selectedFile}
            >
              Submit
            </button>
          </div>
          <div>{aiLoadingText}</div>
          <MarkdownRenderer content={aiResponse} />
        </div>
        <div
          ref={stepperRef}
          className="stepper stepper-links d-flex flex-column pt-15"
          id="kt_create_account_stepper"
        >
          <div className="stepper-nav mb-5">
            <div className="stepper-item current" data-kt-stepper-element="nav">
              <h3 className="stepper-title">Project Type</h3>
            </div>

            <div className="stepper-item" data-kt-stepper-element="nav">
              <h3 className="stepper-title">Project Initial</h3>
            </div>

            <div className="stepper-item" data-kt-stepper-element="nav">
              <h3 className="stepper-title">Business Info</h3>
            </div>

            <div className='stepper-item' data-kt-stepper-element='nav'>
              <h3 className='stepper-title'>Billing Details</h3>
            </div>

            <div className='stepper-item' data-kt-stepper-element='nav'>
              <h3 className='stepper-title'>Completed</h3>
            </div>
          </div>

          <Formik validationSchema={currentSchema} initialValues={initValues} onSubmit={submitStep}>
            {() => (
              <Form className='mx-auto mw-600px w-100 pt-15 pb-10' id='kt_create_account_form'>
                <div className='current' data-kt-stepper-element='content'>
                  <Step1 />
                </div>

                <div data-kt-stepper-element='content'>
                  <Step2 />
                </div>

                <div data-kt-stepper-element='content'>
                  <Step3 />
                </div>

                <div data-kt-stepper-element='content'>
                  <Step4 />
                </div>

                <div data-kt-stepper-element='content'>
                  <Step5 />
                </div>

                <div className='d-flex flex-stack pt-15'>
                  <div className='mr-2'>
                    <button
                      onClick={prevStep}
                      type='button'
                      className='btn btn-lg btn-light-primary me-3'
                      data-kt-stepper-action='previous'
                    >
                      <KTSVG
                        path='/media/icons/duotune/arrows/arr063.svg'
                        className='svg-icon-4 me-1'
                      />
                      Back
                    </button>
                  </div>

                  <div>
                    <button type='submit' className='btn btn-lg btn-primary me-3'>
                      <span className='indicator-label'>
                        {!isSubmitButton && 'Continue'}
                        {isSubmitButton && 'Submit'}
                        <KTSVG
                          path='/media/icons/duotune/arrows/arr064.svg'
                          className='svg-icon-3 ms-2 me-0'
                        />
                      </span>
                    </button>
                  </div>
                </div>
              </Form>
            )}
          </Formik>
        </div>
      </div>
    </div>
  )
}

export {GptNewProject}
